/**************************************************************************** 
** File: tosping.c
**
** Author: Michael S. Borella <mike_borella@mw.3com.com>
**
** TOSping client
**
*****************************************************************************/

#include <stdio.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <netdb.h>
#include "tosping.h"
#include "parse_cl.h"
#include "ip.h"

#define HOSTNAME_SIZE 64

struct arg_t *my_args;     /* Command line arguments */
extern char version[];


/*----------------------------------------------------------------------------
 *
 * main()
 *
 *----------------------------------------------------------------------------
 */

int main(int argc, char *argv[])
{
  const int one = 1;
  int s, n;
  IPHdr iph;
  char hostname[HOSTNAME_SIZE];
  char buf[32];
  char *desthost;
  struct in_addr in;
  struct sockaddr_in sin;    /* an Internet endpoint address         */
  int srv_rec_tos, srv_snt_tos;
  void syserr(char *,...);
  u_int32_t host2ip(char *);
  void alarm_handler(void);

  /*
   * Announce who we are
   */

  printf("%s v%s  3Com Advanced Technologies Research Center\n", 
          argv[0], version);

  /*
   * Parse command line
   */
  
  my_args = parse_cl(argc, argv);
  if (my_args->optind <= argc-1)
    desthost = argv[my_args->optind];
  else
    {
      usage(argv[0]);
      exit(1); /* NOT REACHED - makes compiler happy */
    }
  
  /*
   * Set up raw socket so that we can specifcy the header (leave root mode)
   */
  
  s = socket(AF_INET, SOCK_RAW, IPPROTO_TOS);
  setuid(getuid());
  if (setsockopt(s, IPPROTO_IP, IP_HDRINCL, (const void *) &one, 
		 sizeof(one)) < 0)
    syserr("Can't put raw socket into header-write mode");

  /*
   * Make the IP header
   */

  iph.ip_hl = 5;
  iph.ip_v = 4;
  iph.ip_tos = my_args->t;
  iph.ip_len = htons(20);
  iph.ip_id = 0; /* tells kernel to make one up */
  iph.ip_off = 0;
  iph.ip_ttl = 255;
  iph.ip_p = IPPROTO_TOS;
  iph.ip_csum = 0; /* kernel should compute this for us */

  /*
   * Put our host down at the sender
   */

  if (gethostname(hostname, HOSTNAME_SIZE) < 0)
    syserr("can't get host name");
  iph.ip_src = host2ip(hostname);

  /*
   * Put down our destination host, then set up data structure for connect,
   * not worrying about port numbers
   */
  
  iph.ip_dst = host2ip(desthost);
  sin.sin_addr.s_addr = (u_long) iph.ip_dst;
  sin.sin_family = AF_INET;

  /*
   * Connect the socket to make reading and writing easier
   */

  if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0)
    syserr("can't connect to %s", desthost);

  /*
   * Set up 5 second alarm timeout in case of lost packets or unreachable
   * hosts.  We can't count on getting ICMP messages on this socket 
   * so this isn't an unreasonable approach...
   */

  signal(SIGALRM, (sighandler_t) alarm_handler);
  alarm(5);

  /*
   * Send the packet
   */

  if (write(s, (char *) &iph, sizeof(iph)) < 0)
    syserr("can't write tosping packet");

#ifdef DEBUG
  {
    struct in_addr addr1, addr2;
    
    addr1.s_addr = iph.ip_src;
    addr2.s_addr = iph.ip_dst;
    printf("DEBUG: Sending a packet with TOS = %d from %s to %s\n", 
	   iph.ip_tos, inet_ntoa(addr1), inet_ntoa(addr2));
  }
#endif

  printf("We sent TOS = %d... ", my_args->t);

  /*
   * read response then turn off alarm
   */

  if ((n = read(s, buf, sizeof(buf))) < 0)
      syserr("read error");
  alarm(0);

  /*
   * get IP header
   */

  memcpy((char *) &iph, buf, sizeof(iph));
  
  /*
   * get the tos byte received and sent by the server
   */
  
  in.s_addr = iph.ip_src;
  sscanf(&buf[sizeof(iph)], "%d:%d", &srv_rec_tos, &srv_snt_tos);
  printf("%s received TOS = %d\n", inet_ntoa(in), srv_rec_tos);
  printf("%s sent TOS = %d... ", inet_ntoa(in), srv_snt_tos);
  printf("We received TOS = %d\n", iph.ip_tos);
  
  return 0;

}


/*----------------------------------------------------------------------------
 *
 * host2ip()
 *
 * Convert a host address string in either domain name or dotted decimal
 * format to a 32-bit ip address in u_long format
 *
 *----------------------------------------------------------------------------
 */

u_int32_t host2ip(char *host)
{
  u_int32_t ipaddr;
  struct hostent *phe;
  void syserr(char *,...);
  
  /*
   * First try domain name format; if that doesn't work, try dotted decimal
   */

  if ((phe = gethostbyname(host)) )
    memcpy((u_int32_t *) &ipaddr, phe->h_addr, phe->h_length);
  else 
    if ((ipaddr = inet_addr(host)) == INADDR_NONE)
      syserr("can't get convert %s to IP address", host);

  return (ipaddr);
}

/*----------------------------------------------------------------------------
 *
 * alarm_handler()
 *
 * Dummy handler function for SIGALRM
 *
 *----------------------------------------------------------------------------
 */

void alarm_handler(void)
{
  printf("No response\n");
  exit(0);
}

